# Django settings for test_project project.

DEBUG = True
TEMPLATE_DEBUG = DEBUG
ADMINS = ()
MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    },
}

# TIME_ZONE = 'America/Chicago'
# LANGUAGE_CODE = 'en-us'
# USE_I18N = True
# USE_L10N = True
# USE_TZ = True
# MEDIA_ROOT = ''
# MEDIA_URL = ''

SECRET_KEY = 'secret'
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
)

ROOT_URLCONF = 'test_project.urls'
WSGI_APPLICATION = 'test_project.wsgi.application'

TEMPLATE_DIRS = ()

INSTALLED_APPS = (
    'templatefield',
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}


from setuptools import setup

setup(
    name="django-templatefield",
    version="0.0.1",
    description="Django model field for saving/retrieving templates for rendering.",
    long_description=open("README.md").read(),
    keywords="django, templates",
    author="Jared Morse",
    author_email="jarcoal@gmail.com",
    url="https://github.com/jarcoal/django-templatefield",
    license="MIT",
    packages=["templatefield"],
    install_requires=['django'],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Environment :: Web Environment",
    ],
)


from django.utils import six
from django.template.loader import get_template_from_string
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.template import Template, TemplateSyntaxError, defaulttags, loader_tags
from django.core.exceptions import ValidationError

# these tags are considered unsafe and can be disabled as a group
UNSAFE_TAGS = (
    defaulttags.DebugNode,
    loader_tags.ExtendsNode,
    loader_tags.IncludeNode,
    defaulttags.CsrfTokenNode,
    defaulttags.LoadNode,
    loader_tags.BlockNode,
    defaulttags.URLNode,
)

class TemplateField(six.with_metaclass(models.SubfieldBase, models.TextField)):
    """
    A field that stores a Django template.
    """

    _errors = {
        'invalid': _('template contains invalid syntax.'),
        'not_allowed': _('template tag not allowed: %s'),
    }

    def __init__(self, allow_unsafe_tags=False, **kwargs):
        self.allow_unsafe_tags = allow_unsafe_tags
        super(TemplateField, self).__init__(**kwargs)

    def validate(self, value, model_instance):
        super(TemplateField, self).validate(value, model_instance)

        if not self.allow_unsafe_tags:
            for node in value.nodelist:
                if isinstance(node, UNSAFE_TAGS):
                    raise ValidationError(self._errors['not_allowed'] % node)

    def get_default(self):
        """defaults can be specified as a string or a django template obj"""

        if self.has_default():
            default = self.default

            if callable(default):
                default = default()

            if isinstance(default, six.string_types):
                return get_template_from_string(default)

            if isinstance(default, Template):
                return default

        return super(TemplateField, self).get_default()

    def to_python(self, value):
        """convert to template object"""

        if isinstance(value, six.string_types):
            try:
                return get_template_from_string(value)
            except TemplateSyntaxError:
                raise ValidationError(self._errors['invalid'])

        return value

    def get_prep_value(self, value):
        """prepare template for the database"""

        if value is None:
            if not self.null and self.blank:
                return ""
            return None

        return value.origin.source

    def value_to_string(self, value):
        return self.get_prep_value(self._get_val_from_obj(value))
